@@ -7,7 +7,7 @@
 static VList *now;
 static VList *before_last;
 
-// 関数プロトタイプ
+/* 関数プロトタイプ */
 static inline VList * append_list(VList *first, const void *tmp) __attribute__((always_inline));
 
 /*******************************************************************************
@@ -15,7 +15,21 @@ static inline VList * append_list(VList *first, const void *tmp) __attribute__((
 *******************************************************************************/
 static inline VList * append_list(VList *first, const void *tmp)
 {
-	if(first != NULL)
+	if(first == NULL)
+	{
+		/* リストに最初の要素を追加する */
+
+		first = xmalloc(sizeof(VList));
+		/*
+		* キャストしないと警告が出る
+		* warning: initialization discards qualifiers from pointer target type
+		*/
+		first->data = (void *)tmp;
+		first->next = NULL;
+		first->prev = NULL;
+		first->tail = first;
+	}
+	else
 	{
 		now = xmalloc(sizeof(VList));
 		before_last = first->tail;
@@ -32,20 +46,6 @@ static inline VList * append_list(VList *first, const void *tmp)
 
 		first->tail = now;
 	}
-	else
-	{
-		// リストに最初の要素を追加する
-
-		first = xmalloc(sizeof(VList));
-		/*
-		* キャストしないと警告が出る
-		* warning: initialization discards qualifiers from pointer target type
-		*/
-		first->data = (void *)tmp;
-		first->next = NULL;
-		first->prev = NULL;
-		first->tail = first;
-	}
 
 	return first;
 }
