@@ -205,6 +205,35 @@ RData * check_target(OPArg *oparg, VList *drive_list, SDir *sdir)
 		exit(EXIT_FAILURE);
 	}
 
+	/* 三項演算子 */
+	((loop_to - loop) == 1) ? (acount = SINGULAR) : (acount = PLURAL);
+
+	switch(acount)
+	{
+	case SINGULAR:
+		{
+			char *c = oparg->argv[loop];
+			if(
+				(c[0] == '.') &&
+				((c[1] == '\0') || ((c[1] == G_DIR_SEPARATOR) && (c[2] == '\0')))
+			)
+			{
+				errno = 0;
+
+				if(mkdir(oparg->argv[loop_to], 0755) == -1)
+				{
+					if((errno != EEXIST) && (errno != 0))
+					{
+						print_error("mkdir", __FILE__, __LINE__, cpdd);
+						fprintf(stderr, "コピー先フォルダの作成に失敗しました\n");
+						exit(EXIT_FAILURE);
+					}
+				}
+			}
+		}
+		break;
+	}
+
 	errno = 0;
 	realpath(oparg->argv[loop_to], tmp_to);
 
@@ -238,9 +267,6 @@ RData * check_target(OPArg *oparg, VList *drive_list, SDir *sdir)
 	int len_to = strlen(tmp_to);
 	delete_last_slash(len_to, tmp_to);
 
-	/* 三項演算子 */
-	((loop_to - loop) == 1) ? (acount = SINGULAR) : (acount = PLURAL);
-
 	for(; loop < loop_to; loop++)
 	{
 		_Bool from_current_flag = false;
@@ -361,12 +387,16 @@ RData * check_target(OPArg *oparg, VList *drive_list, SDir *sdir)
 			if(acount == PLURAL)
 			{
 				char *target;
+				errno = 0;
 
 				if(mkdir(oparg->argv[loop_to], 0755) == -1)
 				{
-					print_error("mkdir", __FILE__, __LINE__, cpdd);
-					fprintf(stderr, "コピー先フォルダの作成に失敗しました\n");
-					exit(EXIT_FAILURE);
+					if((errno != EEXIST) && (errno != 0))
+					{
+						print_error("mkdir", __FILE__, __LINE__, cpdd);
+						fprintf(stderr, "コピー先フォルダの作成に失敗しました\n");
+						exit(EXIT_FAILURE);
+					}
 				}
 				else
 				{
