@@ -76,7 +76,7 @@
 
 #define MD5_FINISH \
 {\
-	md5_finish(&state, digest);\
+	md5_finish(state, digest);\
 	char *hex_output = xmalloc((16 * 2) + 1);\
 \
 	for(int di = 0; di < 16; ++di)\
@@ -113,12 +113,16 @@ static long psize;
 static _Bool from_open;
 static _Bool from_stat;
 static _Bool to_open;
+static _Bool verify;
 
 /* ※注意。グローバル変数。 */
 char *write_now;
 
 /* 関数プロトタイプ */
 void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd);
+inline static void mw_small(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
+inline static void mw_big(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
+inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify);
 static void * read_thread_func(void *unknown);
 static void * do_md5_append(void *tmp);
 
@@ -129,7 +133,6 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 	static struct stat stat_buf;
 	static _Bool filesystem_check = false;
 	static _Bool fallocate_support = false;
-	static _Bool verify = false;
 	static _Bool verify_option_check = false;
 
 	from_stat = false;
@@ -344,6 +347,8 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 		static struct verify_data thread_argument;
 		static pthread_t t_verify;
 
+		digest[0] = '\0';
+
 		if(verify == true)
 		{
 			md5_init(&state);
@@ -358,267 +363,15 @@ void read_write(const OPArg *oparg, const SDir *sdir, CPInfo *cpinfo, CPDD *cpdd
 		/* 1048576 == 1024 * 1024 == 1MB */
 		if(1048576 > from_size_local)
 		{
-			errno = 0;
-			buf[0] = mmap(NULL, from_size_local, PROT_READ, MAP_PRIVATE, ifrom, 0);
-
-			if(buf[0] == MAP_FAILED)
-			{
-				perror("mmap");
-			}
-			else
-			{
-				total_read_local = from_size_local;
-
-				errno = 0;
-				if(madvise(buf[0], from_size_local, MADV_SEQUENTIAL) == -1)
-				{
-					print_error("madvise", __FILE__, __LINE__, cpdd);
-				}
-
-				if(verify == true)
-				{
-					thread_argument.state = &state;
-					thread_argument.buffer = buf[0];
-					thread_argument.len = from_size_local;
-
-					pthread_create(&t_verify, NULL, do_md5_append, (void *)&thread_argument);
-				}
-
-				char *tmp = buf[0];
-				const char * const endp = tmp + from_size_local;
-
-				while(tmp < endp)
-				{
-					errno = 0;
-					if((written_size[0] = write(ito, tmp, endp - tmp)) == -1)
-					{
-						print_error("write", __FILE__, __LINE__, cpdd);
-						break;
-					}
-					tmp += written_size[0];
-					total_write_local += written_size[0];
-				}
-
-				if(verify == true)
-				{
-					pthread_join(t_verify, NULL);
-
-					MD5_FINISH
-				}
-
-				errno = 0;
-				if(munmap(buf[0], from_size_local) != 0)
-				{
-					print_error("munmap", __FILE__, __LINE__, cpdd);
-				}
-			}
+			mw_small(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
 		}
 		else if(cpinfo->copy_mode == DIFFERENT)
 		{
-			cpdd->thread_mode_count++;
-			{
-				VList *b_size_list = oparg->b_size_list;
-				b_size_list = b_size_list->tail;
-
-				int *i = (int *)b_size_list->data;
-				int low = *i;
-
-				b_size_list = b_size_list->prev;
-				int high;
-
-				for(;;)
-				{
-					i = (int *)b_size_list->data;
-					high = *i;
-
-					if(high > from_size_local)
-					{
-						buffer_size_thread = low;
-						break;
-					}
-					else if(b_size_list->prev == NULL)
-					{
-						buffer_size_thread = buffer_size_half;
-						break;
-					}
-					else
-					{
-						low = high;
-						b_size_list = b_size_list->prev;
-					}
-				}
-			}
-
-			rbuf_num = 0;
-			wbuf_num = 1;
-
-			pthread_t t_read;
-			pthread_create(&t_read, NULL, read_thread_func, (void *)&rbuf_num);
-			pthread_join(t_read, NULL);
-
-			int num_change;
-
-			for(;;)
-			{
-				num_change = rbuf_num;
-				rbuf_num = wbuf_num;
-				wbuf_num = num_change;
-
-				pthread_create(&t_read, NULL, read_thread_func, (void *)&rbuf_num);
-
-				if(verify == true)
-				{
-					thread_argument.state = &state;
-					thread_argument.buffer = buf[wbuf_num];
-					thread_argument.len = next_write_size[wbuf_num];
-
-					pthread_create(&t_verify, NULL, do_md5_append, (void *)&thread_argument);
-				}
-
-				if(next_write_size[wbuf_num] > 0)
-				{
-					char *tmp = (char *)buf[wbuf_num];
-					const char * const endp = tmp + next_write_size[wbuf_num];
-
-					while(tmp < endp)
-					{
-						errno = 0;
-						if((written_size[wbuf_num] = write(ito, tmp, endp - tmp)) == -1)
-						{
-							print_error("write", __FILE__, __LINE__, cpdd);
-							total_write_error++;
-							break;
-						}
-
-						tmp += written_size[wbuf_num];
-						total_write_local += written_size[wbuf_num];
-					}
-				}
-				else
-				{
-					written_size[wbuf_num] = 0;
-				}
-
-				if(verify == true)
-				{
-					pthread_join(t_verify, NULL);
-				}
-
-				if(next_write_size[wbuf_num] > 0)
-				{
-					errno = 0;
-					if(munmap(buf[wbuf_num], next_write_size[wbuf_num]) == -1)
-					{
-						print_error("munmap", __FILE__, __LINE__, cpdd);
-						total_write_error++;
-					}
-				}
-
-				pthread_join(t_read, NULL);
-
-				if((from_size_local <= total_write_local) || (written_size[wbuf_num] <= 0))
-				{
-					break;
-				}
-			}
-
-			if(verify == true)
-			{
-				MD5_FINISH
-			}
+			mw_thread(oparg, cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
 		}
 		else
 		{
-			for(;;)
-			{
-				off_t read_size = from_size_local - total_read_local;
-
-				if(read_size >= buffer_size)
-				{
-					read_size = buffer_size;
-				}
-				else
-				{
-					off_t tmp = read_size % psize;
-
-					if(tmp != read_size)
-					{
-						read_size = read_size - tmp;
-					}
-				}
-
-				errno = 0;
-				buf[0] = mmap(NULL, read_size, PROT_READ, MAP_PRIVATE, ifrom, total_read_local);
-
-				if(buf[0] == MAP_FAILED)
-				{
-					perror("mmap");
-					break;
-				}
-				else
-				{
-					total_read_local += read_size;
-
-					errno = 0;
-					if(madvise(buf[0], read_size, MADV_SEQUENTIAL) == -1)
-					{
-						print_error("madvise", __FILE__, __LINE__, cpdd);
-					}
-
-					if(verify == true)
-					{
-						thread_argument.state = &state;
-						thread_argument.buffer = buf[0];
-						thread_argument.len = read_size;
-
-						pthread_create(&t_verify, NULL, do_md5_append, (void *)&thread_argument);
-					}
-
-					char *tmp = buf[0];
-					const char * const endp = tmp + read_size;
-
-					while(tmp < endp)
-					{
-						errno = 0;
-						if((written_size[0] = write(ito, tmp, endp - tmp)) == -1)
-						{
-							print_error("write", __FILE__, __LINE__, cpdd);
-
-							errno = 0;
-							if(munmap(buf[0], read_size) != 0)
-							{
-								print_error("munmap", __FILE__, __LINE__, cpdd);
-							}
-
-							goto LOOP_END;
-						}
-						tmp += written_size[0];
-						total_write_local += written_size[0];
-					}
-
-					if(verify == true)
-					{
-						pthread_join(t_verify, NULL);
-					}
-
-					errno = 0;
-					if(munmap(buf[0], read_size) != 0)
-					{
-						print_error("munmap", __FILE__, __LINE__, cpdd);
-					}
-				}
-
-				if(total_read_local >= from_size_local)
-				{
-					break;
-				}
-			}
-LOOP_END:
-;
-			if(verify == true)
-			{
-				MD5_FINISH
-			}
+			mw_big(cpinfo, cpdd, &state, digest, &thread_argument, &t_verify);
 		}
 	}
 	else if((from_size_local == 0) && (to_open == true))
@@ -706,14 +459,7 @@ LOOP_END:
 		case FIVE:
 			HASH_CHECK
 			break;
-/*
-		case MEMCMP:
-			if((cpinfo->from_size > 0) && (cpinfo->to_size > 0))
-			{
-				compare_memcmp(oparg, cpinfo, sdir, cpdd);
-			}
-			break;
-*/
+
 		case V_FIVE:
 			if(cpinfo->verify_md5 == NULL)
 			{
@@ -753,6 +499,280 @@ LOOP_END:
 	}
 }
 
+/*******************************************************************************
+*******************************************************************************/
+inline static void mw_small(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+{
+	errno = 0;
+	buf[0] = mmap(NULL, from_size_local, PROT_READ, MAP_PRIVATE, ifrom, 0);
+
+	if(buf[0] == MAP_FAILED)
+	{
+		perror("mmap");
+	}
+	else
+	{
+		total_read_local = from_size_local;
+
+		errno = 0;
+		if(madvise(buf[0], from_size_local, MADV_SEQUENTIAL) == -1)
+		{
+			print_error("madvise", __FILE__, __LINE__, cpdd);
+		}
+
+		if(verify == true)
+		{
+			thread_argument->state = state;
+			thread_argument->buffer = buf[0];
+			thread_argument->len = from_size_local;
+
+			pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
+		}
+
+		char *tmp = buf[0];
+		const char * const endp = tmp + from_size_local;
+
+		while(tmp < endp)
+		{
+			errno = 0;
+			if((written_size[0] = write(ito, tmp, endp - tmp)) == -1)
+			{
+				print_error("write", __FILE__, __LINE__, cpdd);
+				break;
+			}
+			tmp += written_size[0];
+			total_write_local += written_size[0];
+		}
+
+		if(verify == true)
+		{
+			pthread_join(*t_verify, NULL);
+
+			MD5_FINISH
+		}
+
+		errno = 0;
+		if(munmap(buf[0], from_size_local) != 0)
+		{
+			print_error("munmap", __FILE__, __LINE__, cpdd);
+		}
+	}
+}
+
+/*******************************************************************************
+*******************************************************************************/
+inline static void mw_big(CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+{
+	for(;;)
+	{
+		off_t read_size = from_size_local - total_read_local;
+
+		if(read_size >= buffer_size)
+		{
+			read_size = buffer_size;
+		}
+		else
+		{
+			off_t tmp = read_size % psize;
+
+			if(tmp != read_size)
+			{
+				read_size = read_size - tmp;
+			}
+		}
+
+		errno = 0;
+		buf[0] = mmap(NULL, read_size, PROT_READ, MAP_PRIVATE, ifrom, total_read_local);
+
+		if(buf[0] == MAP_FAILED)
+		{
+			perror("mmap");
+			break;
+		}
+		else
+		{
+			total_read_local += read_size;
+
+			errno = 0;
+			if(madvise(buf[0], read_size, MADV_SEQUENTIAL) == -1)
+			{
+				print_error("madvise", __FILE__, __LINE__, cpdd);
+			}
+
+			if(verify == true)
+			{
+				thread_argument->state = state;
+				thread_argument->buffer = buf[0];
+				thread_argument->len = read_size;
+
+				pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
+			}
+
+			char *tmp = buf[0];
+			const char * const endp = tmp + read_size;
+
+			while(tmp < endp)
+			{
+				errno = 0;
+				if((written_size[0] = write(ito, tmp, endp - tmp)) == -1)
+				{
+					print_error("write", __FILE__, __LINE__, cpdd);
+
+					errno = 0;
+					if(munmap(buf[0], read_size) != 0)
+					{
+						print_error("munmap", __FILE__, __LINE__, cpdd);
+					}
+
+					goto LOOP_END;
+				}
+				tmp += written_size[0];
+				total_write_local += written_size[0];
+			}
+
+			if(verify == true)
+			{
+				pthread_join(*t_verify, NULL);
+			}
+
+			errno = 0;
+			if(munmap(buf[0], read_size) != 0)
+			{
+				print_error("munmap", __FILE__, __LINE__, cpdd);
+			}
+		}
+
+		if(total_read_local >= from_size_local)
+		{
+			break;
+		}
+	}
+LOOP_END:
+;
+	if(verify == true)
+	{
+		MD5_FINISH
+	}
+}
+
+/*******************************************************************************
+*******************************************************************************/
+inline static void mw_thread(const OPArg *oparg, CPInfo *cpinfo, CPDD *cpdd, md5_state_t *state, md5_byte_t digest[16], struct verify_data *thread_argument, pthread_t *t_verify)
+{
+	cpdd->thread_mode_count++;
+
+	{
+		VList *b_size_list = oparg->b_size_list;
+		b_size_list = b_size_list->tail;
+
+		int *i = (int *)b_size_list->data;
+		int low = *i;
+
+		b_size_list = b_size_list->prev;
+		int high;
+
+		for(;;)
+		{
+			i = (int *)b_size_list->data;
+			high = *i;
+
+			if(high > from_size_local)
+			{
+				buffer_size_thread = low;
+				break;
+			}
+			else if(b_size_list->prev == NULL)
+			{
+				buffer_size_thread = buffer_size_half;
+				break;
+			}
+			else
+			{
+				low = high;
+				b_size_list = b_size_list->prev;
+			}
+		}
+	}
+
+	rbuf_num = 0;
+	wbuf_num = 1;
+
+	static pthread_t t_read;
+	pthread_create(&t_read, NULL, read_thread_func, (void *)&rbuf_num);
+	pthread_join(t_read, NULL);
+
+	int num_change;
+
+	for(;;)
+	{
+		num_change = rbuf_num;
+		rbuf_num = wbuf_num;
+		wbuf_num = num_change;
+
+		pthread_create(&t_read, NULL, read_thread_func, (void *)&rbuf_num);
+
+		if(verify == true)
+		{
+			thread_argument->state = state;
+			thread_argument->buffer = buf[wbuf_num];
+			thread_argument->len = next_write_size[wbuf_num];
+
+			pthread_create(t_verify, NULL, do_md5_append, (void *)thread_argument);
+		}
+
+		if(next_write_size[wbuf_num] > 0)
+		{
+			char *tmp = (char *)buf[wbuf_num];
+			const char * const endp = tmp + next_write_size[wbuf_num];
+
+			while(tmp < endp)
+			{
+				errno = 0;
+				if((written_size[wbuf_num] = write(ito, tmp, endp - tmp)) == -1)
+				{
+					print_error("write", __FILE__, __LINE__, cpdd);
+					total_write_error++;
+					break;
+				}
+
+				tmp += written_size[wbuf_num];
+				total_write_local += written_size[wbuf_num];
+			}
+		}
+		else
+		{
+			written_size[wbuf_num] = 0;
+		}
+
+		if(verify == true)
+		{
+			pthread_join(*t_verify, NULL);
+		}
+
+		if(next_write_size[wbuf_num] > 0)
+		{
+			errno = 0;
+			if(munmap(buf[wbuf_num], next_write_size[wbuf_num]) == -1)
+			{
+				print_error("munmap", __FILE__, __LINE__, cpdd);
+				total_write_error++;
+			}
+		}
+
+		pthread_join(t_read, NULL);
+
+		if((from_size_local <= total_write_local) || (written_size[wbuf_num] <= 0))
+		{
+			break;
+		}
+	}
+
+	if(verify == true)
+	{
+		MD5_FINISH
+	}
+}
+
 /*******************************************************************************
  * 読み取り関数。引数は使用しない。
 *******************************************************************************/
